@@ -23,7 +23,7 @@ package org.apache.catalina;
  * Global constants that are applicable to multiple packages within Catalina.
  *
  * @author Craig R. McClanahan
- * @version $Id: Globals.java 1558828 2014-01-16 15:12:59Z markt $
+ *
  */
 
 public final class Globals {
@@ -362,6 +362,16 @@ public final class Globals {
             "org.apache.jasper.XML_VALIDATE_TLD";
 
 
+    /**
+     * Name of the ServletContext init-param that determines if the JSP engine
+     * should validate web.xml files when parsing them.
+     * <p>
+     * This must be kept in sync with org.apache.jasper.Constants
+     */
+    public static final String JASPER_XML_VALIDATION_INIT_PARAM =
+            "org.apache.jasper.XML_VALIDATE";
+
+
     /**
      * Name of the ServletContext init-param that determines if the JSP engine
      * will block external entities from being used in *.tld, *.jspx, *.tagx and
