@@ -27,7 +27,6 @@ import javax.el.ELException;
 import javax.el.ExpressionFactory;
 import javax.el.FunctionMapper;
 import javax.servlet.jsp.tagext.FunctionInfo;
-import javax.servlet.jsp.tagext.JspFragment;
 import javax.servlet.jsp.tagext.PageData;
 import javax.servlet.jsp.tagext.TagAttributeInfo;
 import javax.servlet.jsp.tagext.TagData;
@@ -542,8 +541,7 @@ class Validator {
             // request-time expression
             throwErrorIfExpression(n, "name", "jsp:param");
             n.setValue(getJspAttribute(null, "value", null, null, n
-                    .getAttributeValue("value"), java.lang.String.class, n,
-                    false));
+                    .getAttributeValue("value"), n, null, false));
             visitBody(n);
         }
 
@@ -560,16 +558,14 @@ class Validator {
             JspUtil.checkAttributes("Include action", n, includeActionAttrs,
                     err);
             n.setPage(getJspAttribute(null, "page", null, null, n
-                    .getAttributeValue("page"), java.lang.String.class, n,
-                    false));
+                    .getAttributeValue("page"), n, null, false));
             visitBody(n);
-        };
+        }
 
         public void visit(Node.ForwardAction n) throws JasperException {
             JspUtil.checkAttributes("Forward", n, forwardActionAttrs, err);
             n.setPage(getJspAttribute(null, "page", null, null, n
-                    .getAttributeValue("page"), java.lang.String.class, n,
-                    false));
+                    .getAttributeValue("page"), n, null, false));
             visitBody(n);
         }
 
@@ -584,7 +580,7 @@ class Validator {
             String value = n.getAttributeValue("value");
 
             n.setValue(getJspAttribute(null, "value", null, null, value,
-                    java.lang.Object.class, n, false));
+                    n, null, false));
 
             boolean valueSpecified = n.getValue() != null;
 
@@ -619,8 +615,7 @@ class Validator {
                 err.jspError(n, "jsp.error.usebean.noSession");
 
             Node.JspAttribute jattr = getJspAttribute(null, "beanName", null,
-                    null, n.getAttributeValue("beanName"),
-                    java.lang.String.class, n, false);
+                    null, n.getAttributeValue("beanName"), n, null, false);
             n.setBeanName(jattr);
             if (className != null && jattr != null)
                 err.jspError(n, "jsp.error.usebean.notBoth");
@@ -657,13 +652,11 @@ class Validator {
                 err.jspError(n, "jsp.error.plugin.nocode");
 
             Node.JspAttribute width = getJspAttribute(null, "width", null,
-                    null, n.getAttributeValue("width"), java.lang.String.class,
-                    n, false);
+                    null, n.getAttributeValue("width"), n, null, false);
             n.setWidth(width);
 
             Node.JspAttribute height = getJspAttribute(null, "height", null,
-                    null, n.getAttributeValue("height"),
-                    java.lang.String.class, n, false);
+                    null, n.getAttributeValue("height"), n, null, false);
             n.setHeight(height);
 
             visitBody(n);
@@ -742,8 +735,8 @@ class Validator {
                         }
                     }
                     jspAttrs[i] = getJspAttribute(null, attrs.getQName(i),
-                            attrs.getURI(i), attrs.getLocalName(i), value,
-                            java.lang.Object.class, n, false);
+                            attrs.getURI(i), attrs.getLocalName(i), value, n,
+                            null, false);
                 }
                 n.setJspAttributes(jspAttrs);
             }
@@ -888,14 +881,13 @@ class Validator {
                 if ("name".equals(attrs.getLocalName(i))) {
                     n.setNameAttribute(getJspAttribute(null, attrs.getQName(i),
                             attrs.getURI(i), attrs.getLocalName(i), attrs
-                                    .getValue(i), java.lang.String.class, n,
-                            false));
+                                    .getValue(i), n, null, false));
                 } else {
                     if (jspAttrIndex < jspAttrSize) {
-                        jspAttrs[jspAttrIndex++] = getJspAttribute(null, attrs
-                                .getQName(i), attrs.getURI(i), attrs
-                                .getLocalName(i), attrs.getValue(i),
-                                java.lang.Object.class, n, false);
+                        jspAttrs[jspAttrIndex++] = getJspAttribute(null,
+                                attrs.getQName(i), attrs.getURI(i),
+                                attrs.getLocalName(i), attrs.getValue(i), n,
+                                null, false);
                     }
                 }
             }
@@ -1059,9 +1051,11 @@ class Validator {
                     pageInfo.isDeferredSyntaxAllowedAsLiteral() ||
                     libraryVersion < 2.1;
 
+                String xmlAttributeValue = attrs.getValue(i);
+
                 ELNode.Nodes el = null;
                 if (!runtimeExpression && !pageInfo.isELIgnored()) {
-                    el = ELParser.parse(attrs.getValue(i),
+                    el = ELParser.parse(xmlAttributeValue,
                             deferredSyntaxAllowedAsLiteral);
                     Iterator<ELNode> nodes = el.iterator();
                     while (nodes.hasNext()) {
@@ -1087,6 +1081,21 @@ class Validator {
 
                 boolean expression = runtimeExpression || elExpression;
 
+                // When attribute is not an expression,
+                // contains its textual value with \$ and \# escaping removed.
+                String textAttributeValue;
+                if (!elExpression && el != null) {
+                    // Should be a single Text node
+                    Iterator<ELNode> it = el.iterator();
+                    if (it.hasNext()) {
+                        textAttributeValue = ((ELNode.Text) it.next())
+                                .getText();
+                    } else {
+                        textAttributeValue = "";
+                    }
+                } else {
+                    textAttributeValue = xmlAttributeValue;
+                }
                 for (int j = 0; tldAttrs != null && j < tldAttrs.length; j++) {
                     if (attrs.getLocalName(i).equals(tldAttrs[j].getName())
                             && (attrs.getURI(i) == null
@@ -1128,7 +1137,7 @@ class Validator {
                                     expectedType = tldAttr.getExpectedTypeName();
                                 }
                                 if (expectedType != null) {
-                                    Class expectedClass = String.class;
+                                    Class<?> expectedClass = String.class;
                                     try {
                                         expectedClass = JspUtil.toClass(expectedType, loader);
                                     } catch (ClassNotFoundException e) {
@@ -1138,18 +1147,18 @@ class Validator {
                                     }
                                     // Check casting
                                     try {
-                                        ELSupport.checkType(attrs.getValue(i), expectedClass);
+                                        ELSupport.checkType(textAttributeValue, expectedClass);
                                     } catch (Exception e) {
                                         err.jspError
                                             (n, "jsp.error.coerce_to_type",
-                                             tldAttr.getName(), expectedType, attrs.getValue(i));
+                                             tldAttr.getName(), expectedType, textAttributeValue);
                                     }
                                 }
 
                                 jspAttrs[i] = new Node.JspAttribute(tldAttr,
                                         attrs.getQName(i), attrs.getURI(i), attrs
                                                 .getLocalName(i),
-                                        attrs.getValue(i), false, null, false);
+                                        textAttributeValue, false, null, false);
                             } else {
 
                                 if (deferred && !tldAttr.isDeferredMethod() && !tldAttr.isDeferredValue()) {
@@ -1162,47 +1171,14 @@ class Validator {
                                     err.jspError(n, "jsp.error.attribute.custom.non_rt_with_expr",
                                             tldAttr.getName());
                                 }
-                                
-                                Class expectedType = String.class;
-                                try {
-                                    String typeStr = tldAttr.getTypeName();
-                                    if (tldAttr.isFragment()) {
-                                        expectedType = JspFragment.class;
-                                    } else if (typeStr != null) {
-                                        expectedType = JspUtil.toClass(typeStr,
-                                                loader);
-                                    }
-                                    if (elExpression) {
-                                        // El expression
-                                        validateFunctions(el, n);
-                                        jspAttrs[i] = new Node.JspAttribute(tldAttr,
-                                                attrs.getQName(i), attrs.getURI(i), 
-                                                attrs.getLocalName(i),
-                                                attrs.getValue(i), false, el, false);
-                                        ELContextImpl ctx = new ELContextImpl();
-                                        ctx.setFunctionMapper(getFunctionMapper(el));
-                                        try {
-                                            jspAttrs[i].validateEL(this.pageInfo.getExpressionFactory(), ctx);
-                                        } catch (ELException e) {
-                                            this.err.jspError(n.getStart(),
-                                                    "jsp.error.invalid.expression", 
-                                                    attrs.getValue(i), e.toString());
-                                        }
-                                    } else {
-                                        // Runtime expression
-                                        jspAttrs[i] = getJspAttribute(tldAttr,
-                                                attrs.getQName(i), attrs.getURI(i),
-                                                attrs.getLocalName(i), attrs
-                                                .getValue(i), expectedType, n,
-                                                false);
-                                    }
-                                } catch (ClassNotFoundException e) {
-                                    err.jspError
-                                        (n, "jsp.error.unknown_attribute_type",
-                                         tldAttr.getName(), tldAttr.getTypeName());
-                                }
+
+                                // EL or Runtime expression
+                                jspAttrs[i] = getJspAttribute(tldAttr,
+                                        attrs.getQName(i), attrs.getURI(i),
+                                        attrs.getLocalName(i),
+                                        xmlAttributeValue, n, el, false);
                             }
-                            
+
                         } else {
                             // Attribute does not accept any expressions.
                             // Make sure its value does not contain any.
@@ -1213,14 +1189,14 @@ class Validator {
                             jspAttrs[i] = new Node.JspAttribute(tldAttr,
                                     attrs.getQName(i), attrs.getURI(i), attrs
                                             .getLocalName(i),
-                                    attrs.getValue(i), false, null, false);
+                                    textAttributeValue, false, null, false);
                         }
                         if (expression) {
                             tagDataAttrs.put(attrs.getQName(i),
                                     TagData.REQUEST_TIME_VALUE);
                         } else {
-                            tagDataAttrs.put(attrs.getQName(i), attrs
-                                    .getValue(i));
+                            tagDataAttrs.put(attrs.getQName(i),
+                                    textAttributeValue);
                         }
                         found = true;
                         break;
@@ -1229,9 +1205,8 @@ class Validator {
                 if (!found) {
                     if (tagInfo.hasDynamicAttributes()) {
                         jspAttrs[i] = getJspAttribute(null, attrs.getQName(i),
-                                attrs.getURI(i), attrs.getLocalName(i), attrs
-                                        .getValue(i), java.lang.Object.class,
-                                n, true);
+                                attrs.getURI(i), attrs.getLocalName(i),
+                                xmlAttributeValue, n, el, true);
                     } else {
                         err.jspError(n, "jsp.error.bad_attribute", attrs
                                 .getQName(i), n.getLocalName());
@@ -1309,10 +1284,13 @@ class Validator {
          * If value is null, checks if there are any NamedAttribute subelements
          * in the tree node, and if so, constructs a JspAttribute out of a child
          * NamedAttribute node.
+         *
+         * @param el EL expression, if already parsed by the caller (so that we
+         *  can skip re-parsing it)
          */
         private Node.JspAttribute getJspAttribute(TagAttributeInfo tai,
                 String qName, String uri, String localName, String value,
-                Class expectedType, Node n, boolean dynamic)
+                Node n, ELNode.Nodes el, boolean dynamic)
                 throws JasperException {
 
             Node.JspAttribute result = null;
@@ -1331,34 +1309,52 @@ class Validator {
                     result = new Node.JspAttribute(tai, qName, uri, localName,
                             value.substring(3, value.length() - 2), true, null,
                             dynamic);
-                } else if (pageInfo.isELIgnored()) {
-                    result = new Node.JspAttribute(tai, qName, uri, localName,
-                            value, false, null, dynamic);
                 } else {
-                    // The attribute can contain expressions but is not a
-                    // scriptlet expression; thus, we want to run it through
-                    // the expression interpreter
+                    if (!pageInfo.isELIgnored()) {
+                        // The attribute can contain expressions but is not a
+                        // scriptlet expression; thus, we want to run it through
+                        // the expression interpreter
 
-                    // validate expression syntax if string contains
-                    // expression(s)
-                    ELNode.Nodes el = ELParser.parse(value, pageInfo
-                            .isDeferredSyntaxAllowedAsLiteral());
-
-                    if (el.containsEL()) {
-
-                        validateFunctions(el, n);
-
-                        if (n.getRoot().isXmlSyntax()) {
-                            // The non-EL elements need to be XML escaped
-                            XmlEscapeNonELVisitor v = new XmlEscapeNonELVisitor();
-                            el.visit(v);
-                            result = new Node.JspAttribute(tai, qName, uri,
-                                    localName, v.getText(), false, el, dynamic);
-                        } else {
-                            result = new Node.JspAttribute(tai, qName, uri,
-                                    localName, value, false, el, dynamic);
+                        // validate expression syntax if string contains
+                        // expression(s)
+                        if (el == null) {
+                            el = ELParser.parse(value,
+                                pageInfo.isDeferredSyntaxAllowedAsLiteral());
                         }
 
+                        if (el.containsEL()) {
+                            validateFunctions(el, n);
+                        } else {
+                            // Get text with \$ and \# escaping removed.
+                            // Should be a single Text node
+                            value = ((ELNode.Text) el.iterator().next())
+                                    .getText();
+                            el = null;
+                        }
+                    }
+
+                    if (n instanceof Node.UninterpretedTag &&
+                            n.getRoot().isXmlSyntax()) {
+                        // Attribute values of uninterpreted tags will have been
+                        // XML un-escaped during parsing. Since these attributes
+                        // are part of an uninterpreted tag the value needs to
+                        // be re-escaped before being included in the output.
+                        // The wrinkle is that the output of any EL must not be
+                        // re-escaped as that must be output as is.
+                        if (el != null) {
+                            XmlEscapeNonELVisitor v = new XmlEscapeNonELVisitor(
+                                    pageInfo.isDeferredSyntaxAllowedAsLiteral());
+                            el.visit(v);
+                            value = v.getText();
+                        } else {
+                            value = xmlEscape(value);
+                        }
+                    }
+
+                    result = new Node.JspAttribute(tai, qName, uri, localName,
+                            value, false, el, dynamic);
+
+                    if (el != null) {
                         ELContextImpl ctx = new ELContextImpl();
                         ctx.setFunctionMapper(getFunctionMapper(el));
 
@@ -1370,10 +1366,6 @@ class Validator {
                                     "jsp.error.invalid.expression", value, e
                                             .toString());
                         }
-
-                    } else {
-                        result = new Node.JspAttribute(tai, qName, uri,
-                                localName, value, false, null, dynamic);
                     }
                 }
             } else {
@@ -1395,9 +1387,16 @@ class Validator {
 
         private static class XmlEscapeNonELVisitor extends ELParser.TextBuilder {
 
+            protected XmlEscapeNonELVisitor(
+                    boolean isDeferredSyntaxAllowedAsLiteral) {
+                super(isDeferredSyntaxAllowedAsLiteral);
+            }
+            
             @Override
             public void visit(Text n) throws JasperException {
-                output.append(xmlEscape(n.getText()));
+                output.append(ELParser.escapeLiteralExpression(
+                        xmlEscape(n.getText()),
+                        isDeferredSyntaxAllowedAsLiteral));
             }
         }
 
@@ -1626,7 +1625,7 @@ class Validator {
                 }
                 start = p + 1;
             }
-            return (String[]) params.toArray(new String[params.size()]);
+            return params.toArray(new String[params.size()]);
         }
 
         private FunctionMapper getFunctionMapper(ELNode.Nodes el)
@@ -1654,7 +1653,7 @@ class Validator {
 
                 public void visit(ELNode.Function n) throws JasperException {
 
-                    Class c = null;
+                    Class<?> c = null;
                     Method method = null;
                     try {
                         c = loader.loadClass(n.getFunctionInfo()
@@ -1667,7 +1666,7 @@ class Validator {
                     }
                     String paramTypes[] = n.getParameters();
                     int size = paramTypes.length;
-                    Class params[] = new Class[size];
+                    Class<?> params[] = new Class[size];
                     int i = 0;
                     try {
                         for (i = 0; i < size; i++) {
